/***
 * Excerpted from "Domain-Driven Design Using Naked Objects",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/dhnako for more book information.
***/
package com.pragprog.dhnako.carserv.dom.customer;

import java.util.ArrayList;
import java.util.List;

import org.nakedobjects.applib.AbstractDomainObject;
import org.nakedobjects.applib.annotation.MaxLength;
import org.nakedobjects.applib.annotation.MemberOrder;
import org.nakedobjects.applib.annotation.MultiLine;
import org.nakedobjects.applib.annotation.Optional;
import org.nakedobjects.applib.util.TitleBuffer;

import com.pragprog.dhnako.carserv.dom.vehicle.Car;

public class Customer extends AbstractDomainObject {

	// {{ Identification
	public String title() {
		final TitleBuffer buf = new TitleBuffer();
		if (getTitle() != null) {
			buf.append(getTitle());
		}
		buf.append(getFirstName()).append(getLastName());
		return buf.toString();
	}

	public String iconName() {
		return getTitle() != null? getTitle().getIconName(): null;
	}

	// }}

	
	// {{ Title
	private Title title;

	@MemberOrder(sequence = "1")
	public Title getTitle() {
		return title;
	}

	public void setTitle(final Title title) {
		this.title = title;
	}
	// }}



	// {{ FirstName
	private String firstName;

	@MaxLength(30)
	@MemberOrder(sequence = "1.3")
	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(final String firstName) {
		this.firstName = firstName;
	}
	// }}

	// {{ LastName
	private String lastName;

	@MaxLength(30)
	@MemberOrder(sequence = "1.2")
	public String getLastName() {
		return lastName;
	}

	public void setLastName(final String lastName) {
		this.lastName = lastName;
	}
	// }}

	// {{ Notes
	private String notes;

    @MultiLine(numberOfLines = 10, preventWrapping = false)
	@Optional
	@MaxLength(255)
	@MemberOrder(sequence = "1.3")
	public String getNotes() {
		return notes;
	}

	public void setNotes(final String notes) {
		this.notes = notes;
	}
	// }}
	
	
	// {{ Cars
	private List<Car> cars = new ArrayList<Car>();

	@MemberOrder(sequence = "1")
	public List<Car> getCars() {
		return cars;
	}
 
	public void setCars(final List<Car> cars) {
		this.cars = cars;
	}

	public void addToCars(final Car car) {
		// check for no-op
		if (car == null || getCars().contains(car)) {
			return;
		}
		// dissociate arg from its current parent (if any).
		car.clearOwningCustomer();
		// associate arg
		car.setOwningCustomer(this);
		getCars().add(car);
		// additional business logic
		onAddToCars(car);
	}

	public void removeFromCars(final Car car) {
		// check for no-op
		if (car == null || !getCars().contains(car)) {
			return;
		}
		// dissociate arg
		car.setOwningCustomer(null);
		getCars().remove(car);
		// additional business logic
		onRemoveFromCars(car);
	}

	protected void onAddToCars(final Car car) {
	}

	protected void onRemoveFromCars(final Car car) {
	}
	
	public String validateAddToCars(final Car car) {
		return getCars().size() >= 3?
				"No more than 3 cars per customer": null;
	}

	public String validateRemoveFromCars(final Car car) {
		return null;
	}


	// }}

}
