/***
 * Excerpted from "Domain-Driven Design Using Naked Objects",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/dhnako for more book information.
***/
package com.pragprog.dhnako.carserv.dom.customer;

import java.util.List;

import org.nakedobjects.applib.AbstractFactoryAndRepository;
import org.nakedobjects.applib.Filter;
import org.nakedobjects.applib.annotation.Debug;
import org.nakedobjects.applib.annotation.Exploration;
import org.nakedobjects.applib.annotation.Named;
import org.nakedobjects.applib.annotation.Optional;
import org.nakedobjects.applib.security.UserMemento;

@Named("Customers")
public class CustomerRepository extends AbstractFactoryAndRepository {

	// {{ Identification
	public String iconName() {
		return "Customer";
	}
	// }}
	
	// {{ Create new (still transient) Customer
	public Customer newCustomer() {
		Customer customer = newTransientInstance(Customer.class);
		return customer;
	}
	// }}

	// {{ findByName
	public List<Customer> findByName(
	        @Optional
	        @Named("Last Name")
	        final String lastName,
	        @Optional
	        @Named("First Name")
	        final String firstName) {
	    return allMatches(Customer.class, new Filter<Customer>() {
	        public boolean accept(final Customer customer) {
	            return matches(customer, firstName, lastName);
	        }});
	}
	public String validateFindByName(final String lastName, final String firstName) {
		if (lastName == null && firstName == null) {
			return "Must specify at least one name";
		}
		return null;
	}
	private static boolean matches(
	       final Customer customer, 
	       final String firstName, final String lastName) {
	    return nullSafeEquals(customer.getFirstName(), firstName) &&
	           nullSafeEquals(customer.getLastName(), lastName);
	}
	private static <T> boolean nullSafeEquals(final T s1, final T s2) {
	    return s1 == null || s2 == null ||
	           s1 != null && s2 != null && s1.equals(s2);
	}
	// }}
	

	// {{ all Customers
	@Exploration
	public List<Customer> allCustomers() {
		return allInstances(Customer.class);
	}
	// }}

	// {{ currentUser
	@Debug
	public UserMemento currentUser() {
		return getUser();
	}
	// }}

	
}
