/***
 * Excerpted from "Domain-Driven Design Using Naked Objects",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material, 
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/dhnako for more book information.
***/
package com.pragprog.dhnako.carserv.dom.vehicle;

import java.util.Arrays;
import java.util.List;

import org.nakedobjects.applib.AbstractDomainObject;
import org.nakedobjects.applib.annotation.Bounded;
import org.nakedobjects.applib.annotation.Immutable;
import org.nakedobjects.applib.annotation.MemberOrder;
import org.nakedobjects.applib.annotation.Named;
import org.nakedobjects.applib.util.TitleBuffer;

@Bounded
@Immutable
public class Model extends AbstractDomainObject {

	
	// {{ Identification
	public String title() {
		final TitleBuffer buf = new TitleBuffer();
		buf.append(getName());
		return buf.toString();
	}
	// }}


	// {{ Name
	private String name;

	@MemberOrder(sequence = "1.2")
	public String getName() {
		return name;
	}

	public void setName(final String name) {
		this.name = name;
	}
	// }}


	// {{ Make
	private Make make;

	@MemberOrder(sequence = "1.1")
	public Make getMake() {
		return make;
	}

	public void setMake(final Make make) {
		this.make = make;
	}

	public void modifyMake(final Make make) {
		Make currentMake = getMake();
		// check for no-op
		if (make == null || currentMake == make) {
			return;
		}
		// delegate to parent to associate
		make.addToModels(this);
		// additional business logic
		onModifyMake(currentMake, make);
	}

	public void clearMake() {
		Make currentMake = getMake();
		// check for no-op
		if (currentMake == null) {
			return;
		}
		// delegate to parent to dissociate
		currentMake.removeFromModels(this);
		// additional business logic
		onClearMake(currentMake);
	}

	protected void onModifyMake(final Make oldMake, final Make newMake) {
	}

	protected void onClearMake(final Make oldMake) {
	}
	// }}

	
	// {{ findCarsWithoutRecentService
	@MemberOrder(sequence = "1")
	public List<Car> findCarsWithoutRecentService(
			@Named("Months since")
			final Integer months) {
		return carRepository.findCarsWithoutRecentService(this, months);
	}
	public Integer default0FindCarsWithoutRecentService() {
		return 12;
	}
	private static final List<Integer> RECENT_SERVICE_MONTHS = 
		Arrays.asList(6, 9, 12, 18, 24, 36);
	public List<Integer> choices0FindCarsWithoutRecentService() {
		return RECENT_SERVICE_MONTHS;
	}
	public String validateFindCarsWithoutRecentService(final Integer months) {
		return RECENT_SERVICE_MONTHS.contains(months)? null: "Select from list";
	}
	// }}



	// {{ injected: CarRepository
	private CarRepository carRepository;

	public void setCarRepository(final CarRepository carRepository) {
		this.carRepository = carRepository;
	}
	// }}







}
